
#include "globals.h"
#include "arch/functions_allarch.h"

/*
 * Function that saves gathered information into cachefile.
 * If you modify this function, don't forget to modify read_info too!
 * 
 */

int save_info(){
	int i;
	
	printf("Saving determined information to cache file. \n");

        if ( (fs = fopen(statusfile,"w")) == NULL ) {
		errormsg = strerror(errno);
		fprintf(stderr, "%s: %s.\n",statusfile,errormsg);
		return 22;
	}

	// So at this moment we have file open for writing

//	// architecture of image (int)
//	i=fprintf(fs,"arch:%d\n",img1.arch);
	// hw architecture of analyzed image (string)
	i=fprintf(fs,"architecture:%s\n",img1.architecture); 
	// operating system of image (int)
	i=fprintf(fs,"os:%d\n",img1.os);
	// operating system of analyzed image (string)
	i=fprintf(fs,"operating_system:%s\n",img1.operating_system); 
	// position of VPT (from beginning of file)
	i=fprintf(fs,"VPT_address:0x%llX\n",img1.VPT_address); 
	// size of image in kilobytes
	i=fprintf(fs,"ram_size:%llu\n",img1.ram_size); 
	// distance between name and list_head in process
	i=fprintf(fs,"ts_hl_comm_dst:%d\n",img1.ts_hl_comm_dst); 
	// distance between name and list_head in module
	i=fprintf(fs,"md_hl_name_dst:%d\n",img1.md_hl_name_dst);
	// linear address of tasks list_head of one of process structure
	i=fprintf(fs,"ts_one_of_lists:0x%llX\n",img1.ts_one_of_lists);
	// =TS_MAGIC_HL_MM_DST; 
	// Distance from "tasks" list_head to "mm" mmstruct
	i=fprintf(fs,"ts_hl_mm_dst:%d\n",img1.ts_hl_mm_dst);
	// =MM_MAGIC_PGD_DST;  
	// Distance from beginning of mm_struct to pgd variable
	i=fprintf(fs,"mm_pgd_dst:%d\n",img1.mm_pgd_dst);
	// linear address of module list_head of one of module structures
	i=fprintf(fs,"md_one_of_lists:0x%llX\n",img1.md_one_of_lists);
	// distance between beginning and sizes in vm_area struct(int)
	i=fprintf(fs,"vm_size_dst:%d\n",img1.vm_size_dst);
	// distance between beginning and list_head in vm_area struct(int))
	i=fprintf(fs,"vm_list_dst:%d\n",img1.vm_list_dst);
	
	i=fprintf(fs,"\n# Keep exact format when modifying cache manually.\n"\
	"# Single bad byte break whole formating. "\
	"(maybe XML in next release;)\n"); 

	i=fclose(fs);
	
	return i;
}

/*
 * Function that reads previously saved information from cachefile
 * If you modify this function, don't forget to modify save_info too!
 * 
 */

int read_info(){
	int i=0;

	printf("Looking for cache_file: \"%s\" ... ",statusfile);fflush(stdout);

        if ( (fs = fopen(statusfile,"r")) == NULL ) {
		dbgprint("read_info: file does not exist.\n");
		//file does not exists => we will perform determination
		return 1;	
	}

	// So at this moment we have file open for reading

//	// architecture of image (int)
//	i+=fscanf(fs,"arch:%d\n",&img1.arch);
	// hw architecture of analyzed image (string)
	i+=fscanf(fs,"architecture:%s\n",img1.architecture); 
	// operating system of image (int)
	i+=fscanf(fs,"os:%d\n",&img1.os);
	// operating system of analyzed image (string)
	i+=fscanf(fs,"operating_system:%s\n",img1.operating_system); 
	// position of VPT (from beginning of file)
	i+=fscanf(fs,"VPT_address:0x%llX\n",&img1.VPT_address); 
	// size of image in kilobytes
	i+=fscanf(fs,"ram_size:%llu\n",&img1.ram_size); 
	// distance between name and list_head in process
	i+=fscanf(fs,"ts_hl_comm_dst:%d\n",&img1.ts_hl_comm_dst); 
	// distance between name and list_head in module
	i+=fscanf(fs,"md_hl_name_dst:%d\n",&img1.md_hl_name_dst);
	// linear address of tasks list_head of one of process structure
	i+=fscanf(fs,"ts_one_of_lists:0x%llX\n",&img1.ts_one_of_lists);
	// =TS_MAGIC_HL_MM_DST; 
	// Distance from "tasks" list_head to "mm" mmstruct
	i+=fscanf(fs,"ts_hl_mm_dst:%d\n",&img1.ts_hl_mm_dst);
	// =MM_MAGIC_PGD_DST;  
	//Distance from beginning of mm_struct to pgd variable
	i+=fscanf(fs,"mm_pgd_dst:%d\n",&img1.mm_pgd_dst);
	// linear address of module list_head of one of module structures
	i+=fscanf(fs,"md_one_of_lists:0x%llX\n",&img1.md_one_of_lists);
	// distance between beginning and sizes in vm_area struct(int)
	i+=fscanf(fs,"vm_size_dst:%d\n",&img1.vm_size_dst);
	// distance between beginning and headlist in vm_area struct(int))
	i+=fscanf(fs,"vm_list_dst:%d\n",&img1.vm_list_dst);

	if (i != FORIANA_CACHE_SIZE){
		printf("read_info: %d elements: bad file format.\n",i);
		fflush(stdout);
		fclose(fs); fs = NULL;
		return 2;
	}	

	// Set pointer to correct functions,
	// according to detected architecture
	// This must be done by checking string;(
	// Index in array can change in various versions.
	
	if (set_arch_from_list(img1.architecture) != 0) {
		printf("read_info: cannot set architecture from cache.\n");
		return 3;
	}

	fclose(fs); fs = NULL;
	printf("ok.\n");
	return 0;
}

