/*
 *	Functions for analysing linux systems
*/

#include "../../globals.h"
#include "../functions_allos.h"
#include "../allos.h"
#include "magic.h"
#include "../../arch/functions_allarch.h"

/*
 * Hook for proc_determine
 *
 * linux_hook_proc_determine (pointer to process address)
 * return 0 on success 
 */

int linux_hook_proc_determine(u_int64_t *process){
	
	return 0;
}

/*
 * Hook to determine aditional information,
 * once process structure is found.
 *
 * linux_hook_proc_determine2 ()
 * return 0 on success 
 */

int linux_hook_proc_determine2(){

	int status;
	dbgprint();

	// Find some userspace process
	if ( (status=lin_find_userspace_process()) !=0) {
		printf("	linux_determine: lin_find_userspace_process "\
			"returned: %d, assuming windows OS.\n", status);
		printf("	linux_determine: Reading of userspace "\
			"addresses will be not available..\n");
		return 2;
	}
	dbgprint();

	// determine distances of VMAs
	if ( (status=lin_find_process_VMA(img1.ts_one_of_lists)) !=0) {
		printf("	linux_determine: lin_find_process_VMA "\
			"returned: %d, terminating.\n",status);
		printf("	linux_determine: Reading of userspace "\
			"addresses will be not available..\n");
		return 4;
	}

	dbgprint();

	// VMAs are determined before process VPT,
	// because finding mm_struct offset inside task_struct is more reliable


	// determine VPT distances
	if ( (status=lin_find_process_VPT(img1.ts_one_of_lists)) !=0) {
		printf("	linux_determine: lin_find_process_VPT "\
			"returned: %d.\n",status);
		printf("	linux_determine: Reading of userspace "\
			"addresses will be not available..\n");
		return 6;
	}
	dbgprint();

	return 0;
}

/*
 * Hook for listing processes
 *
 * At this point, os should be no more "unknown"
 * But lets assume it is.
 *
 * linux(pointer to VMA switch)
 * return 0 on success 
 */

int linux_hook_list_processes(int *list_areas){

	if (do_areas_list) {
		// enable area listing
		*list_areas=1;
	}

	return 0;
}

/*
 * Hook for matching process structure
 *
 * linux_hook_verify_task_struct ( 
	pointer to hl size, 
	index in hl list,
	ver_addr (needed for bsd list,
	pointer to bsd list start)
 * return 0 on success 
 */

int linux_hook_verify_task_struct(int *hl_size, int i, u_int64_t ver_addr, u_int64_t *bsd_list_start, int *found_os) {

	int cyc_hl_size=0;

	// Cyclic (Lin/Win) style list_head 
	cyc_hl_size=lin_list_head_size(img1.ts_headlists[i]);

	if ( cyc_hl_size >= *hl_size) {
		// more and not loop (loop < 0)
		/* Sometimes, there are multiple pointers to list_head
		   we are looking for.
		   Therefore our used offset can be wrong, 
		   and we should count string for each element of list.
		 */

		dbgprint("Saving local maximum:(1.try) 0x%llX",
				img1.ts_headlists[i]);

		if ( cyc_hl_size > *hl_size ){

			dbgprint("Saving local maximum: 0x%llX, circ_hl: %d",
				 img1.ts_headlists[i], cyc_hl_size);

			//save which list_head is right one
			img1.ts_headlists_number=i; 

			// remember local maximum
			*hl_size=cyc_hl_size;		
		}
	}
	return 0;
}

/*
 * Hook for matching process structure
 *
 * linux_hook_verify_task_struct2 ( 
 *	found_os, start of non-cyclic hl, pointer to hl )
 *
 * return 0 on success 
 */

int linux_hook_verify_task_struct2(int found_os, u_int64_t bsd_list_start, u_int64_t *list_start) {

	return 0;

}

/*
 * Function that sets pointers to all other 
 * os dependent functions and hooks
 *
 * linux_init (pointer to os structure)
 * return 0 on success 
 */

int linux_init(struct os_struct *self){

	strncpy(self->operating_system,"linux\0",OS_NAME_LEN);
	strncpy(self->magic_process_name,LINUX_MAGIC_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_user_process_name,LINUX_MAGIC_USER_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_module_name,LINUX_MAGIC_MODULE,MD_NAME_LEN);

	self->hook_proc_determine=(int (*)())linux_hook_proc_determine;
	self->hook_proc_determine2=(int (*)())linux_hook_proc_determine2;
	self->hook_list_processes=(int (*)())linux_hook_list_processes;
	self->hook_verify_task_struct=(int (*)())linux_hook_verify_task_struct;
	self->hook_verify_task_struct2=(int (*)())linux_hook_verify_task_struct2;

	self->ts_magic_headlist_number = LINUX_TS_MAGIC_HEADLIST_NUMBER;
	self->ts_magic_headlist_spread = LINUX_TS_MAGIC_HEADLIST_SPREAD;
	self->ts_magic_headlist_front_number = LINUX_TS_MAGIC_HEADLIST_FRONT_NUMBER;
	self->ts_magic_min_list_size = LINUX_TS_MAGIC_MIN_LIST_SIZE;
	self->ts_magic_max_tries = LINUX_TS_MAGIC_MAX_TRIES;
	self->ts_magic_hl_mm_dst_p = LINUX_TS_MAGIC_HL_MM_DST_P;
	self->ts_magic_list_max_err = LINUX_TS_MAGIC_LIST_MAX_ERR;
	self->ts_magic_headlist_max = LINUX_TS_MAGIC_HEADLIST_MAX;
	self->mm_magic_pgd_dst_p = LINUX_MM_MAGIC_PGD_DST_P;
	self->md_magic_headlist_number = LINUX_MD_MAGIC_HEADLIST_NUMBER;
	self->md_magic_max_tries = LINUX_MD_MAGIC_MAX_TRIES;
	self->md_magic_hl_name_dst_p = LINUX_MD_MAGIC_HL_NAME_DST_P;
	self->md_magic_headlist_max = LINUX_MD_MAGIC_HEADLIST_MAX;
	self->md_pattern_spread = LINUX_MD_PATTERN_SPREAD;
	self->max_list_size = LINUX_MAX_LIST_SIZE;


	return 0;
}
