/*
 *	Functions for analysing windows systems
*/

#include "../../globals.h"
#include "../functions_allos.h"
#include "../allos.h"
#include "magic.h"


/*
 * Hook for proc_determine
 *
 * windows_hook_proc_determine (pointer to process address)
 * return 0 on success 
 */

int windows_hook_proc_determine(u_int64_t *process){
	
	return 0;
}



/*
 * Hook to determine aditional information,
 * once process structure is found.
 *
 * windows_hook_proc_determine2 ()
 * return 0 on success 
 */

int windows_hook_proc_determine2(){

	return 0;
}

/*
 * Hook for listing processes
 *
 * For now, control only VMA listing
 * Another hook may be needed for future.
 *
 * windows_hook_list_processes (pointer to VMA switch)
 * return 0 on success 
 */

int windows_hook_list_processes(int *list_areas){

	if (do_areas_list) {
		printf("Disabling VMA listing, "\
			"not implemented for windows kernels.\n");
		*list_areas=0;
	}

	return 0;
}


#if 0 

This is not needed, linux function is used instead 
/*
 * Function that ...
 *
 * windows_hook_verify_task_struct ()
 * return 0 on success 
 */

int windows_hook_verify_task_struct(){

	return 0;
}


/*
 * Hook for matching process structure
 *
 * windows_hook_verify_task_struct2 ( 
 *	found_os, start of non-cyclic hl, pointer to hl )
 *
 * return 0 on success 
 */

int windows_hook_verify_task_struct2(int found_os, u_int64_t bsd_list_start, u_int64_t *list_start) {

	return 0;

}
#endif

/*
 * Function that sets pointers to all other 
 * os dependent functions and hooks
 *
 * windows_init (pointer to os structure)
 * return 0 on success 
 */

int windows_init(struct os_struct *self){

	strncpy(self->operating_system,"windows\0",OS_NAME_LEN);

	strncpy(self->magic_process_name,WINDOWS_MAGIC_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_user_process_name,WINDOWS_MAGIC_USER_PROCESS,TS_COMM_LEN);
	strncpy(self->magic_module_name,WINDOWS_MAGIC_MODULE,MD_NAME_LEN);


	self->hook_proc_determine=(int (*)())windows_hook_proc_determine;
	self->hook_proc_determine2=(int (*)())windows_hook_proc_determine2;
	self->hook_list_processes=(int (*)())windows_hook_list_processes;
	// LINUX used here
	self->hook_verify_task_struct=(int (*)())linux_hook_verify_task_struct;
	self->hook_verify_task_struct2=(int (*)())linux_hook_verify_task_struct2;

	self->ts_magic_headlist_number = WINDOWS_TS_MAGIC_HEADLIST_NUMBER;
	self->ts_magic_headlist_spread = WINDOWS_TS_MAGIC_HEADLIST_SPREAD;
	self->ts_magic_headlist_front_number = WINDOWS_TS_MAGIC_HEADLIST_FRONT_NUMBER;
	self->ts_magic_min_list_size = WINDOWS_TS_MAGIC_MIN_LIST_SIZE;
	self->ts_magic_max_tries = WINDOWS_TS_MAGIC_MAX_TRIES;
	self->ts_magic_hl_mm_dst_p = WINDOWS_TS_MAGIC_HL_MM_DST_P;
	self->ts_magic_list_max_err = WINDOWS_TS_MAGIC_LIST_MAX_ERR;
	self->ts_magic_headlist_max = WINDOWS_TS_MAGIC_HEADLIST_MAX;
	self->mm_magic_pgd_dst_p = WINDOWS_MM_MAGIC_PGD_DST_P;
	self->md_magic_headlist_number = WINDOWS_MD_MAGIC_HEADLIST_NUMBER;
	self->md_magic_max_tries = WINDOWS_MD_MAGIC_MAX_TRIES;
	self->md_magic_hl_name_dst_p = WINDOWS_MD_MAGIC_HL_NAME_DST_P;
	self->md_magic_headlist_max = WINDOWS_MD_MAGIC_HEADLIST_MAX;
	self->md_pattern_spread = WINDOWS_MD_PATTERN_SPREAD;
	self->max_list_size = WINDOWS_MAX_LIST_SIZE;


	return 0;
}
